@@ -45,9 +45,9 @@ startSrcDest (src:dest:[])
 	| otherwise = notAnnexed src $ do
 		showStart "reinject" dest
 		next $ ifAnnexed dest
-			(perform src)
+			(\key -> perform src key (verifyKeyContent DefaultVerify UnVerified key src))
 			stop
-startSrcDest _ = error "specify a src file and a dest file"
+startSrcDest _ = giveup "specify a src file and a dest file"
 
 startKnown :: FilePath -> CommandStart
 startKnown src = notAnnexed src $ do
@@ -56,23 +56,24 @@ startKnown src = notAnnexed src $ do
 	case mkb of
 		Nothing -> error "Failed to generate key"
 		Just (key, _) -> ifM (isKnownKey key)
-			( next $ perform src key
+			( next $ perform src key (return True)
 			, do
 				warning "Not known content; skipping"
 				next $ next $ return True
 			)
 
 notAnnexed :: FilePath -> CommandStart -> CommandStart
-notAnnexed src = ifAnnexed src (error $ "cannot used annexed file as src: " ++ src)
+notAnnexed src = ifAnnexed src $
+	giveup $ "cannot used annexed file as src: " ++ src
 
-perform :: FilePath -> Key -> CommandPerform
-perform src key = ifM move
+perform :: FilePath -> Key -> Annex Bool -> CommandPerform
+perform src key verify = ifM move
 	( next $ cleanup key
 	, error "failed"
 	)
   where
 	move = checkDiskSpaceToGet key False $
-		ifM (verifyKeyContent DefaultVerify UnVerified key src)
+		ifM verify
 			( do
 				moveAnnex key src
 				return True
