@@ -1,12 +1,14 @@
 {- git-annex output messages
  -
- - Copyright 2010-2014 Joey Hess <id@joeyh.name>
+ - Copyright 2010-2016 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
 module Messages (
 	showStart,
+	ActionItem,
+	mkActionItem,
 	showStart',
 	showNote,
 	showAction,
@@ -25,6 +27,7 @@ module Messages (
 	earlyWarning,
 	warningIO,
 	indent,
+	JSON.JSONChunk(..),
 	maybeShowJSON,
 	showFullJSON,
 	showCustom,
@@ -37,9 +40,9 @@ module Messages (
 	commandProgressDisabled,
 	outputMessage,
 	implicitMessage,
+	withMessageState,
 ) where
 
-import Text.JSON
 import System.Log.Logger
 import System.Log.Formatter
 import System.Log.Handler (setFormatter)
@@ -48,18 +51,22 @@ import System.Log.Handler.Simple
 import Common
 import Types
 import Types.Messages
+import Types.ActionItem
 import Messages.Internal
 import qualified Messages.JSON as JSON
-import Types.Key
 import qualified Annex
 
 showStart :: String -> FilePath -> Annex ()
-showStart command file = outputMessage (JSON.start command (Just file) Nothing) $
+showStart command file = outputMessage json $
 	command ++ " " ++ file ++ " "
+  where
+	json = JSON.start command (Just file) Nothing
 
-showStart' :: String -> Key -> Maybe FilePath -> Annex ()
-showStart' command key afile = outputMessage (JSON.start command afile (Just key)) $
-	command ++ " " ++ fromMaybe (key2file key) afile ++ " "
+showStart' :: String -> Key -> ActionItem -> Annex ()
+showStart' command key i = outputMessage json $
+	command ++ " " ++ actionItemDesc i key ++ " "
+  where
+	json = JSON.start command (actionItemWorkTreeFile i) (Just key)
 
 showNote :: String -> Annex ()
 showNote s = outputMessage (JSON.note s) $ "(" ++ s ++ ") "
@@ -77,7 +84,7 @@ showSideAction m = Annex.getState Annex.output >>= go
 			Annex.changeState $ \s -> s { Annex.output = st' }
 		| sideActionBlock st == InBlock = return ()
 		| otherwise = p
-	p = outputMessage q $ "(" ++ m ++ "...)\n"
+	p = outputMessage JSON.none $ "(" ++ m ++ "...)\n"
 			
 showStoringStateAction :: Annex ()
 showStoringStateAction = showSideAction "recording state in git"
@@ -102,7 +109,7 @@ doSideAction' b a = do
 {- Make way for subsequent output of a command. -}
 showOutput :: Annex ()
 showOutput = unlessM commandProgressDisabled $
-	outputMessage q "\n"
+	outputMessage JSON.none "\n"
 
 showLongNote :: String -> Annex ()
 showLongNote s = outputMessage (JSON.note s) ('\n' : indent s ++ "\n")
@@ -132,7 +139,7 @@ earlyWarning = warning' False
 warning' :: Bool -> String -> Annex ()
 warning' makeway w = do
 	when makeway $
-		outputMessage q "\n"
+		outputMessage JSON.none "\n"
 	outputError (w ++ "\n")
 
 {- Not concurrent output safe. -}
@@ -145,19 +152,13 @@ warningIO w = do
 indent :: String -> String
 indent = intercalate "\n" . map (\l -> "  " ++ l) . lines
 
-{- Shows a JSON fragment only when in json mode. -}
-maybeShowJSON :: JSON a => [(String, a)] -> Annex ()
-maybeShowJSON v = withOutputType $ liftIO . go
-  where
-	go JSONOutput = JSON.add v
-	go _ = return ()
+{- Shows a JSON chunk only when in json mode. -}
+maybeShowJSON :: JSON.JSONChunk v -> Annex ()
+maybeShowJSON v = void $ withMessageState $ outputJSON (JSON.add v)
 
 {- Shows a complete JSON value, only when in json mode. -}
-showFullJSON :: JSON a => [(String, a)] -> Annex Bool
-showFullJSON v = withOutputType $ liftIO . go
-  where
-	go JSONOutput = JSON.complete v >> return True
-	go _ = return False
+showFullJSON :: JSON.JSONChunk v -> Annex Bool
+showFullJSON v = withMessageState $ outputJSON (JSON.complete v)
 
 {- Performs an action that outputs nonstandard/customized output, and
  - in JSON mode wraps its output in JSON.start and JSON.end, so it's
@@ -171,10 +172,10 @@ showCustom command a = do
 	outputMessage (JSON.end r) ""
 
 showHeader :: String -> Annex ()
-showHeader h = outputMessage q $ (h ++ ": ")
+showHeader h = outputMessage JSON.none $ (h ++ ": ")
 
 showRaw :: String -> Annex ()
-showRaw s = outputMessage q (s ++ "\n")
+showRaw s = outputMessage JSON.none (s ++ "\n")
 
 setupConsole :: IO ()
 setupConsole = do
@@ -208,11 +209,11 @@ debugEnabled = do
 {- Should commands that normally output progress messages have that
  - output disabled? -}
 commandProgressDisabled :: Annex Bool
-commandProgressDisabled = withOutputType $ \t -> return $ case t of
-	QuietOutput -> True
-	JSONOutput -> True
-	NormalOutput -> False
-	ConcurrentOutput {} -> True
+commandProgressDisabled = withMessageState $ \s -> return $
+	case outputType s of
+		QuietOutput -> True
+		JSONOutput _ -> True
+		NormalOutput -> concurrentOutputEnabled s
 
 {- Use to show a message that is displayed implicitly, and so might be
  - disabled when running a certian command that needs more control over its
