@@ -27,6 +27,7 @@ export UPGRADE_LOCATION
 #export FORCE_GIT_VERSION
 
 # Uncomment to get rid of cabal installed libraries.
+#cabal list --installed
 #rm -rf /c/Users/jenkins/AppData/Roaming/cabal /c/Users/jenkins/AppData/Roaming/ghc
 
 # Don't allow build artifact from a past successful build to be extracted
@@ -38,9 +39,11 @@ rm -f git-annex-installer.exe
 # for haskell libraries to link them with the cygwin library.
 cabal update || true
 
-cabal install warp-3.1.3.1
-
-cabal install --only-dependencies || true
+cabal install --only-dependencies \
+		--constraint='persistent-sqlite ==2.2' \
+		--constraint='cryptonite ==0.7' \
+		--constraint='mwc-random ==0.13.3.2' \
+		|| true
 
 # Detect when the last build was an incremental build and failed, 
 # and try a full build. Done this way because this shell seems a bit
